import 'dart:math';

import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';
import 'package:ocg_app/src/widgets/ease_in_widget.dart';

class BottomNavbar extends StatelessWidget {
  final List<BottomNavbarItem> items;
  final Color activeColor;
  final Color color;
  final Color middleColor;
  final int current;
  final Function(int index, bool isMiddle) onTap;
  final double toolbarHeight;
  const BottomNavbar(
      {Key key,
      this.current = 0,
      @required this.items,
      this.onTap,
      this.toolbarHeight = 56,
      this.color = Colors.black54,
      this.activeColor = Colors.black,
      this.middleColor = Colors.white})
      : assert(items != null),
        assert(onTap != null),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      height: toolbarHeight,
      margin: EdgeInsets.symmetric(horizontal: 8, vertical: 12),
      padding: EdgeInsets.symmetric(vertical: 8),
      decoration: BoxDecoration(
          border: Border.all(color: Colors.black12, width: 0.4),
          color: Colors.white,
          boxShadow: [
            BoxShadow(
                color: Colors.black.withOpacity(0.02),
                blurRadius: 12,
                spreadRadius: 0.01)
          ],
          borderRadius: BorderRadius.circular(32)),
      child: LayoutBuilder(
        builder: (context, constraints) {
          double width = constraints.biggest.width;
          List<BottomNavbarItem> actions = [
            ...items,
          ];

          double iconWidth = width / actions.length;
          double iconHeight = min(toolbarHeight / 2, 24);
          int middle = actions.length ~/ 2;
          return Row(
            children: actions
                .map((e) => Expanded(
                        child: _buildItem(e, iconHeight, iconWidth,
                            isActive: e == actions[current], onTap: (item) {
                      int index = items.indexOf(item);
                      onTap(index, index == middle);
                    }, isMiddle: e == actions[middle])))
                .toList(),
          );
        },
      ),
    );
  }

  Widget _buildItem(BottomNavbarItem item, double height, double width,
      {Function(BottomNavbarItem item) onTap,
      bool isMiddle = false,
      bool isActive}) {
    if (isMiddle) {
      return EaseInWidget(
        onTap: onTap == null ? null : () => onTap(item),
        child: Container(
          decoration: BoxDecoration(
            color: activeColor,
            shape: BoxShape.circle,
          ),
          alignment: Alignment.center,
          child: Icon(
            item.iconData,
            color: middleColor,
            size: min(width, height) * 1.1,
          ),
        ),
      );
    }
    return InkWell(
      onTap: onTap == null ? null : () => onTap(item),
      child: Container(
        width: width,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Icon(
              item.iconData,
              color: isActive ? activeColor : color,
              size: min(width, height),
            ),
            Text(
              "${item.label}",
              style: TextStyle(
                fontSize: 11,
                color: isActive ? activeColor : color,
              ),
            )
          ],
        ),
      ),
    );
  }
}

class BottomNavbarItem extends Equatable {
  final String label;
  final IconData iconData;

  BottomNavbarItem(this.label, this.iconData);

  @override
  List<Object> get props => [
        label,
      ];
}
